@@ -27,7 +27,6 @@
 #include <gtk/gtk.h>
 #include <telepathy-logger/log-manager.h>
 
-#include <libempathy/empathy-call-factory.h>
 #include <libempathy/empathy-contact-manager.h>
 #include <libempathy/empathy-dispatcher.h>
 #include <libempathy/empathy-utils.h>
@@ -40,6 +39,9 @@
 #include "empathy-contact-dialogs.h"
 #include "empathy-ui-utils.h"
 #include "empathy-share-my-desktop.h"
+#include "empathy-call-utils.h"
+
+static GtkWidget *empathy_contact_block_menu_item_new (EmpathyContact *);
 
 GtkWidget *
 empathy_contact_menu_new (EmpathyContact             *contact,
@@ -138,6 +140,19 @@ empathy_contact_menu_new (EmpathyContact             *contact,
 		gtk_widget_show (item);
 	}
 
+	/* Separator & Block */
+	if (features & EMPATHY_CONTACT_FEATURE_BLOCK &&
+	    (item = empathy_contact_block_menu_item_new (contact)) != NULL) {
+		GtkWidget *sep;
+
+		sep = gtk_separator_menu_item_new ();
+		gtk_menu_shell_append (shell, sep);
+		gtk_widget_show (sep);
+
+		gtk_menu_shell_append (shell, item);
+		gtk_widget_show (item);
+	}
+
 	return menu;
 }
 
@@ -212,6 +227,84 @@ empathy_contact_add_menu_item_new (EmpathyContact *contact)
 	return item;
 }
 
+static void
+empathy_contact_block_menu_item_toggled (GtkCheckMenuItem *item,
+					 EmpathyContact   *contact)
+{
+	static guint block_signal = 0;
+	EmpathyContactManager *manager;
+	gboolean blocked, abusive;
+
+	if (block_signal > 0)
+		return;
+
+	blocked = gtk_check_menu_item_get_active (item);
+
+	if (blocked) {
+		/* confirm the user really wishes to block the contact */
+		GtkWidget *parent;
+
+		/* gtk_menu_get_attach_widget() doesn't behave properly here
+		 * for some reason */
+		parent = g_object_get_data (
+			G_OBJECT (gtk_widget_get_parent (GTK_WIDGET (item))),
+			"window");
+
+		if (!empathy_block_contact_dialog_show (GTK_WINDOW (parent),
+					contact, &abusive))
+			return;
+	}
+
+	manager = empathy_contact_manager_dup_singleton ();
+	empathy_contact_list_set_blocked (EMPATHY_CONTACT_LIST (manager),
+					  contact, blocked, abusive);
+	g_object_unref (manager);
+
+	/* update the toggle with the blocked status */
+	block_signal++;
+	gtk_check_menu_item_set_active (item, blocked);
+	block_signal--;
+}
+
+static GtkWidget *
+empathy_contact_block_menu_item_new (EmpathyContact *contact)
+{
+	GtkWidget *item;
+	EmpathyContactManager *manager;
+	TpConnection *connection;
+	EmpathyContactListFlags flags;
+	gboolean blocked;
+
+	g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), NULL);
+
+	if (!empathy_contact_manager_initialized ()) {
+		return NULL;
+	}
+
+	manager = empathy_contact_manager_dup_singleton ();
+	connection = empathy_contact_get_connection (contact);
+
+	flags = empathy_contact_manager_get_flags_for_connection (manager,
+			connection);
+
+	if (!(flags & EMPATHY_CONTACT_LIST_CAN_BLOCK)) {
+		return NULL;
+	}
+
+	item = gtk_check_menu_item_new_with_mnemonic (_("_Block Contact"));
+	blocked = empathy_contact_list_get_blocked (
+			EMPATHY_CONTACT_LIST (manager),
+			contact);
+
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item), blocked);
+
+	g_signal_connect (item, "toggled",
+			G_CALLBACK (empathy_contact_block_menu_item_toggled),
+			contact);
+
+	return item;
+}
+
 static void
 empathy_contact_chat_menu_item_activated (GtkMenuItem *item,
 	EmpathyContact *contact)
@@ -231,6 +324,7 @@ empathy_contact_chat_menu_item_new (EmpathyContact *contact)
 	image = gtk_image_new_from_icon_name (EMPATHY_IMAGE_MESSAGE,
 					      GTK_ICON_SIZE_MENU);
 	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+	gtk_widget_set_sensitive (item, !empathy_contact_is_user (contact));
 	gtk_widget_show (image);
 
 	g_signal_connect (item, "activate",
@@ -245,8 +339,10 @@ empathy_contact_audio_call_menu_item_activated (GtkMenuItem *item,
 	EmpathyContact *contact)
 {
 
-	empathy_call_factory_new_call_with_streams (contact, TRUE, FALSE,
-		gtk_get_current_event_time (), NULL);
+	empathy_call_new_with_streams (empathy_contact_get_id (contact),
+		empathy_contact_get_account (contact),
+		TRUE, FALSE,
+		gtk_get_current_event_time ());
 }
 
 GtkWidget *
@@ -261,7 +357,8 @@ empathy_contact_audio_call_menu_item_new (EmpathyContact *contact)
 	image = gtk_image_new_from_icon_name (EMPATHY_IMAGE_VOIP,
 					      GTK_ICON_SIZE_MENU);
 	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
-	gtk_widget_set_sensitive (item, empathy_contact_can_voip_audio (contact));
+	gtk_widget_set_sensitive (item, empathy_contact_can_voip_audio (contact) &&
+					!empathy_contact_is_user (contact));
 	gtk_widget_show (image);
 
 	g_signal_connect (item, "activate",
@@ -275,8 +372,10 @@ static void
 empathy_contact_video_call_menu_item_activated (GtkMenuItem *item,
 	EmpathyContact *contact)
 {
-	empathy_call_factory_new_call_with_streams (contact, TRUE, TRUE,
-		gtk_get_current_event_time (), NULL);
+	empathy_call_new_with_streams (empathy_contact_get_id (contact),
+		empathy_contact_get_account (contact),
+		TRUE, TRUE,
+		gtk_get_current_event_time ());
 }
 
 GtkWidget *
@@ -291,7 +390,8 @@ empathy_contact_video_call_menu_item_new (EmpathyContact *contact)
 	image = gtk_image_new_from_icon_name (EMPATHY_IMAGE_VIDEO_CALL,
 					      GTK_ICON_SIZE_MENU);
 	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
-	gtk_widget_set_sensitive (item, empathy_contact_can_voip_video (contact));
+	gtk_widget_set_sensitive (item, empathy_contact_can_voip_video (contact) &&
+					!empathy_contact_is_user (contact));
 	gtk_widget_show (image);
 
 	g_signal_connect (item, "activate",
@@ -313,6 +413,7 @@ GtkWidget *
 empathy_contact_log_menu_item_new (EmpathyContact *contact)
 {
 	TplLogManager     *manager;
+	TplEntity         *entity;
 	gboolean           have_log;
 	GtkWidget         *item;
 	GtkWidget         *image;
@@ -320,11 +421,15 @@ empathy_contact_log_menu_item_new (EmpathyContact *contact)
 	g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), NULL);
 
 	manager = tpl_log_manager_dup_singleton ();
-	have_log = tpl_log_manager_exists (manager,
-					       empathy_contact_get_account (contact),
-					       empathy_contact_get_id (contact),
-					       FALSE);
+	entity = tpl_entity_new_from_tp_contact (empathy_contact_get_tp_contact (contact),
+						 TPL_ENTITY_CONTACT);
 
+	have_log = tpl_log_manager_exists (manager,
+					   empathy_contact_get_account (contact),
+					   entity,
+					   TPL_EVENT_MASK_TEXT);
+
+	g_object_unref (entity);
 	g_object_unref (manager);
 
 	item = gtk_image_menu_item_new_with_mnemonic (_("_Previous Conversations"));
@@ -352,7 +457,8 @@ empathy_contact_file_transfer_menu_item_new (EmpathyContact *contact)
 	item = gtk_image_menu_item_new_with_mnemonic (_("Send File"));
 	image = gtk_image_new_from_icon_name (EMPATHY_IMAGE_DOCUMENT_SEND,
 					      GTK_ICON_SIZE_MENU);
-	gtk_widget_set_sensitive (item, empathy_contact_can_send_files (contact));
+	gtk_widget_set_sensitive (item, empathy_contact_can_send_files (contact) &&
+					!empathy_contact_is_user (contact));
 	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
 	gtk_widget_show (image);
 
@@ -363,7 +469,6 @@ empathy_contact_file_transfer_menu_item_new (EmpathyContact *contact)
 	return item;
 }
 
-/* FIXME  we should check if the contact supports vnc stream tube */
 GtkWidget *
 empathy_contact_share_my_desktop_menu_item_new (EmpathyContact *contact)
 {
@@ -577,6 +682,12 @@ empathy_contact_invite_menu_item_new (EmpathyContact *contact)
 					      GTK_ICON_SIZE_MENU);
 	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
 
+	if (empathy_contact_is_user (contact)) {
+		gtk_widget_set_sensitive (item, FALSE);
+		gtk_widget_show (image);
+		return item;
+	}
+
 	mgr = empathy_chatroom_manager_dup_singleton (NULL);
 	rooms = empathy_chatroom_manager_get_chatrooms (mgr,
 		empathy_contact_get_account (contact));
