@@ -38,7 +38,6 @@
 #include <folks/folks.h>
 #include <folks/folks-telepathy.h>
 
-#include <libempathy/empathy-call-factory.h>
 #include <libempathy/empathy-individual-manager.h>
 #include <libempathy/empathy-contact-groups.h>
 #include <libempathy/empathy-dispatcher.h>
@@ -48,6 +47,7 @@
 #include "empathy-individual-menu.h"
 #include "empathy-individual-store.h"
 #include "empathy-contact-dialogs.h"
+#include "empathy-individual-dialogs.h"
 #include "empathy-images.h"
 #include "empathy-linking-dialog.h"
 #include "empathy-cell-renderer-expander.h"
@@ -157,11 +157,7 @@ individual_view_tooltip_destroy_cb (GtkWidget *widget,
 {
   EmpathyIndividualViewPriv *priv = GET_PRIV (view);
 
-  if (priv->tooltip_widget != NULL)
-    {
-      DEBUG ("Tooltip destroyed");
-      tp_clear_object (&priv->tooltip_widget);
-    }
+  tp_clear_object (&priv->tooltip_widget);
 }
 
 static gboolean
@@ -850,8 +846,6 @@ individual_view_popup_menu_idle_cb (gpointer user_data)
       gtk_widget_show (menu);
       gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, data->button,
           data->time);
-      g_object_ref_sink (menu);
-      g_object_unref (menu);
     }
 
   g_slice_free (MenuPopupData, data);
@@ -1620,6 +1614,7 @@ individual_view_is_visible_individual (EmpathyIndividualView *self,
   EmpathyLiveSearch *live = EMPATHY_LIVE_SEARCH (priv->search_widget);
   const gchar *str;
   GList *personas, *l;
+  gboolean is_favorite, contains_interesting_persona = FALSE;
 
   /* We're only giving the visibility wrt filtering here, not things like
    * presence. */
@@ -1629,8 +1624,24 @@ individual_view_is_visible_individual (EmpathyIndividualView *self,
       return FALSE;
     }
 
+  /* Hide all individuals which consist entirely of uninteresting personas */
+  personas = folks_individual_get_personas (individual);
+  for (l = personas; l; l = l->next)
+    {
+      if (empathy_folks_persona_is_interesting (FOLKS_PERSONA (l->data)))
+        {
+          contains_interesting_persona = TRUE;
+          break;
+        }
+    }
+
+  if (contains_interesting_persona == FALSE)
+    return FALSE;
+
+  is_favorite = folks_favouritable_get_is_favourite (
+      FOLKS_FAVOURITABLE (individual));
   if (is_searching == FALSE)
-    return (priv->show_offline || is_online);
+    return (priv->show_offline || is_online || is_favorite);
 
   /* check alias name */
   str = folks_aliasable_get_alias (FOLKS_ALIASABLE (individual));
@@ -1639,14 +1650,13 @@ individual_view_is_visible_individual (EmpathyIndividualView *self,
     return TRUE;
 
   /* check contact id, remove the @server.com part */
-  personas = folks_individual_get_personas (individual);
   for (l = personas; l; l = l->next)
     {
       const gchar *p;
       gchar *dup_str = NULL;
       gboolean visible;
 
-      if (!TPF_IS_PERSONA (l->data))
+      if (!empathy_folks_persona_is_interesting (FOLKS_PERSONA (l->data)))
         continue;
 
       str = folks_persona_get_display_id (l->data);
@@ -1913,7 +1923,7 @@ individual_view_dispose (GObject *object)
 
   tp_clear_object (&priv->store);
   tp_clear_object (&priv->filter);
-  tp_clear_pointer (&priv->tooltip_widget, gtk_widget_destroy);
+  tp_clear_object (&priv->tooltip_widget);
 
   empathy_individual_view_set_live_search (view, NULL);
 
@@ -2213,16 +2223,22 @@ empathy_individual_view_get_selected_group (EmpathyIndividualView *view,
   return name;
 }
 
-static gboolean
+static int
 individual_view_remove_dialog_show (GtkWindow *parent,
     const gchar *message,
-    const gchar *secondary_text)
+    const gchar *secondary_text,
+    gboolean block_button)
 {
   GtkWidget *dialog;
   gboolean res;
 
   dialog = gtk_message_dialog_new (parent, GTK_DIALOG_MODAL,
       GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE, "%s", message);
+
+  if (block_button)
+    gtk_dialog_add_button (GTK_DIALOG (dialog),
+        _("Delete and Block"), GTK_RESPONSE_REJECT);
+
   gtk_dialog_add_buttons (GTK_DIALOG (dialog),
       GTK_STOCK_CANCEL, GTK_RESPONSE_NO,
       GTK_STOCK_DELETE, GTK_RESPONSE_YES, NULL);
@@ -2234,7 +2250,7 @@ individual_view_remove_dialog_show (GtkWindow *parent,
   res = gtk_dialog_run (GTK_DIALOG (dialog));
   gtk_widget_destroy (dialog);
 
-  return (res == GTK_RESPONSE_YES);
+  return res;
 }
 
 static void
@@ -2254,7 +2270,7 @@ individual_view_group_remove_activate_cb (GtkMenuItem *menuitem,
           group);
       parent = empathy_get_toplevel_window (GTK_WIDGET (view));
       if (individual_view_remove_dialog_show (parent, _("Removing group"),
-              text))
+              text, FALSE) == GTK_RESPONSE_YES)
         {
           EmpathyIndividualManager *manager =
               empathy_individual_manager_dup_singleton ();
@@ -2333,26 +2349,46 @@ individual_view_remove_activate_cb (GtkMenuItem *menuitem,
 
   if (individual != NULL)
     {
+      EmpathyIndividualManager *manager;
       gchar *text;
       GtkWindow *parent;
+      gboolean can_block;
+      int res;
+
+      manager = empathy_individual_manager_dup_singleton ();
+      can_block = empathy_individual_manager_supports_blocking (manager,
+          individual);
 
       parent = empathy_get_toplevel_window (GTK_WIDGET (view));
       text =
-          g_strdup_printf (_
-          ("Do you really want to remove the contact '%s'?"),
+          g_strdup_printf (
+          _("Do you really want to remove the contact '%s'?"),
           folks_aliasable_get_alias (FOLKS_ALIASABLE (individual)));
-      if (individual_view_remove_dialog_show (parent, _("Removing contact"),
-              text))
+      res = individual_view_remove_dialog_show (parent, _("Removing contact"),
+              text, can_block);
+      if (res == GTK_RESPONSE_YES || res == GTK_RESPONSE_REJECT)
         {
-          EmpathyIndividualManager *manager;
+          gboolean abusive;
+
+          if (res == GTK_RESPONSE_REJECT &&
+              empathy_block_individual_dialog_show (parent, individual,
+                &abusive))
+            {
+              empathy_individual_manager_set_blocked (manager, individual,
+                  TRUE, abusive);
+            }
+          else
+            {
+              goto finally;
+            }
 
-          manager = empathy_individual_manager_dup_singleton ();
           empathy_individual_manager_remove (manager, individual, "");
-          g_object_unref (G_OBJECT (manager));
         }
 
+finally:
       g_free (text);
       g_object_unref (individual);
+      g_object_unref (manager);
     }
 }
 
