@@ -43,7 +43,14 @@ uri_regex_dup_singleton (void)
 
 	/* We intentionally leak the regex so it's not recomputed */
 	if (!uri_regex) {
-		uri_regex = g_regex_new (URI_REGEX, 0, 0, NULL);
+		GError *error = NULL;
+
+		uri_regex = g_regex_new (URI_REGEX, 0, 0, &error);
+		if (uri_regex == NULL) {
+			g_warning ("Failed to create reg exp: %s", error->message);
+			g_error_free (error);
+			return NULL;
+		}
 	}
 
 	return g_regex_ref (uri_regex);
@@ -75,6 +82,11 @@ empathy_string_match_link (const gchar *text,
 	gint        last = 0;
 
 	uri_regex = uri_regex_dup_singleton ();
+	if (uri_regex == NULL) {
+		empathy_string_parser_substr (text, len, sub_parsers, user_data);
+		return;
+	}
+
 	match = g_regex_match_full (uri_regex, text, len, 0, 0, &match_info, NULL);
 	if (match) {
 		gint s = 0, e = 0;
@@ -186,7 +198,7 @@ empathy_string_replace_escaped (const gchar *text,
 	GString *string = user_data;
 	gchar *escaped;
 	guint i;
-	gssize escaped_len, old_len;
+	gsize escaped_len, old_len;
 
 	escaped = g_markup_escape_text (text, len);
 	escaped_len = strlen (escaped);
@@ -217,14 +229,6 @@ empathy_add_link_markup (const gchar *text)
 
 	g_return_val_if_fail (text != NULL, NULL);
 
-	/* GtkLabel with links could make infinite loop because of
-	 * GNOME bug #612066. It is fixed in GTK >= 2.18.8 and GTK >= 2.19.7.
-	 * FIXME: Remove this check once we have an hard dep on GTK 2.20 */
-	if (gtk_check_version (2, 18, 8) != NULL ||
-	    (gtk_minor_version == 19 && gtk_micro_version < 7)) {
-		return g_markup_escape_text (text, -1);
-	}
-
 	string = g_string_sized_new (strlen (text));
 	empathy_string_parser_substr (text, -1, parsers, string);
 
