@@ -117,6 +117,7 @@ account_manager_ready_for_accounts_cb (GObject *source_object,
       if ((account = tp_account_new (bus, account_path, &error)))
         {
           tp_account_prepare_async (account, NULL, account_prepare_cb, manager);
+          g_object_unref (bus);
           return;
         }
       else
@@ -222,10 +223,12 @@ main (int argc, char *argv[])
 
   if (unique_app_is_running (unique_app))
     {
-      unique_app_send_message (unique_app, UNIQUE_ACTIVATE, NULL);
-
-      g_object_unref (unique_app);
-      return EXIT_SUCCESS;
+      if (unique_app_send_message (unique_app, UNIQUE_ACTIVATE, NULL) ==
+          UNIQUE_RESPONSE_OK)
+        {
+          g_object_unref (unique_app);
+          return EXIT_SUCCESS;
+        }
     }
 
   account_manager = tp_account_manager_dup ();
