@@ -403,6 +403,10 @@ chat_window_contact_menu_update (EmpathyChatWindowPriv *priv,
 
 	if (orig_submenu == NULL || !gtk_widget_get_visible (orig_submenu)) {
 		submenu = empathy_chat_get_contact_menu (priv->current_chat);
+
+		/* gtk_menu_attach_to_widget() doesn't behave nicely here */
+		g_object_set_data (G_OBJECT (submenu), "window", priv->dialog);
+
 		gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu), submenu);
 		gtk_widget_show (menu);
 	} else {
@@ -1300,7 +1304,7 @@ chat_window_show_or_update_notification (EmpathyChatWindow *window,
 		   to an existing notification with the same title.
 		   In this way the previous message will not be lost: the new
 		   message will appear below it, in the same notification */
-		notification = notify_notification_new (header, escaped, NULL, NULL);
+		notification = notify_notification_new (header, escaped, NULL);
 
 		if (priv->notification == NULL) {
 			priv->notification = notification;
@@ -1351,6 +1355,7 @@ chat_window_set_highlight_room_tab_label (EmpathyChat *chat)
 static void
 chat_window_new_message_cb (EmpathyChat       *chat,
 			    EmpathyMessage    *message,
+			    gboolean pending,
 			    EmpathyChatWindow *window)
 {
 	EmpathyChatWindowPriv *priv;
@@ -1425,7 +1430,11 @@ chat_window_new_message_cb (EmpathyChat       *chat,
 
 		empathy_sound_play (GTK_WIDGET (priv->dialog),
 		    EMPATHY_SOUND_MESSAGE_INCOMING);
-		chat_window_show_or_update_notification (window, message, chat);
+
+		/* Pending messages have already been displayed in the approver, so we don't
+		* display a notification for those. */
+		if (!pending)
+			chat_window_show_or_update_notification (window, message, chat);
 	}
 
 	/* update the number of unread messages and the window icon */
